/////////////////////////////////////////////////////////////////////////////
// $Id:$
//////////////////////////////////////////////////////////////////////////////
//                                Stella 96
//                        "An Atari 2600 Simulator"
//                         Copyright (c) 1995,1996
//                            Bradford W. Mott
//////////////////////////////////////////////////////////////////////////////
//
// main
//
//   Main loop of the program
//
// Bradford W. Mott
// May 13,1996
//
//////////////////////////////////////////////////////////////////////////////
// $Log:$
//////////////////////////////////////////////////////////////////////////////

#include <time.h>
#include <stdio.h>
#include <string.h>
#include <strstream.h>
#include "termos2.hxx"
#include "bspf.hxx"
#include "Console.hxx"
#include "DefProps.hxx"
#include "Event.hxx"
#include "MediaSrc.hxx"
#include "PropsSet.hxx"
#include "sndos2.hxx"
#include "System.hxx"


#define INCL_WINSTDFILE   // OS/2 include for File Dialog
#define INCL_WINDIALOGS   // OS/2 include for Window Dialog
#include <os2.h>

extern int newgame = FALSE;
extern int firstgame = 0;
char* StellaHelp;
char* StellaIni;
char* StellaPro;

// Pointer to the console object or the null pointer
Console* theConsole;
// Event object to use
Event theEvent;

bool theQuitIndicator = false;


void setupProperties(PropertiesSet& set)
{
  // Try to load the file $HOME/.stella.pro file
//  string filename = "stella.pro";

  ifstream stream(StellaPro, ios::in | ios::nocreate | ios::binary);
  if(stream)
  {
    // File was opened so load properties from it
    set.load(stream, &Console::defaultProperties());
  }
  else
  {
    // Couldn't open the file so use the builtin properties file
    strstream builtin;
    for(const char** p = defaultPropertiesFile(); *p != 0; ++p)
    {
      builtin << *p << endl;
    }
    set.load(builtin, &Console::defaultProperties());
  }
}



//////////////////////////////////////////////////////////////////////////////
// Main loop
//////////////////////////////////////////////////////////////////////////////
main(int argc, char* argv[])
{
   char filename[CCHMAXPATH] = "";
   char* justfilename;

   uInt32 size;
   uInt8* image;
   ifstream in;

   memset(filename,0, CCHMAXPATH);

   int i;

   StellaHelp = (char*) malloc(CCHMAXPATH);
   memset(StellaHelp, 0, CCHMAXPATH);

   char* separator = strrchr(argv[0], BSPF_PATH_SEPARATOR);

   if(separator != 0)
   {
      strncpy(StellaHelp, argv[0], separator - argv[0]);
      strcat(StellaHelp, "\\stella.inf");
   }
   else
      strcpy(StellaHelp, "stella.inf");

// find the OS/2 INI file for Stella
    StellaIni = (char*) malloc(CCHMAXPATH);
    memset(StellaIni, 0, CCHMAXPATH);

    separator = strrchr(argv[0], BSPF_PATH_SEPARATOR);

    if(separator != 0)
    {
       strncpy(StellaIni, argv[0], separator - argv[0]);
       strcat(StellaIni, "\\stella.ini");
    }
    else
       strcpy(StellaIni, "stella.ini");

    StellaPro = (char*) malloc(CCHMAXPATH);
    memset(StellaPro, 0, CCHMAXPATH);

    separator = strrchr(argv[0], BSPF_PATH_SEPARATOR);

    if(separator != 0)
    {
       strncpy(StellaPro, argv[0], separator - argv[0]);
       strcat(StellaPro, "\\stella.pro");
    }
    else
       strcpy(StellaPro, "stella.pro");

   for (i = 0;i<argc;i++)
         firstgame = i;

// if (firstgame != 0)
//   strcpy(filename, argv[firstgame]);

   if (OpenWindow() == FALSE) return 1;

   DosSetCurrentDir("ROMS");   /* default into ROMS directory if present */
   PropertiesSet propertiesSet("Cartridge.Name");
   setupProperties(propertiesSet);

   SoundOS2 sound;

   for(;;)        // loop till no cartridge
   {
      if (GetCartridge(filename) == FALSE)          // display file dialog to get cartridge
      {
         newgame = FALSE;
         goto quitstella;                            // end if no cartridge
      }

      in.open(filename, ios::in | ios::nocreate | ios::binary);
      image = new uInt8[512 * 1024];
      in.read(image, 512 * 1024);
      size = in.gcount();
      in.close();

      // Get just the filename of the file containing the ROM image
      justfilename = (!strrchr(filename, BSPF_PATH_SEPARATOR)) ? filename : strrchr(filename, BSPF_PATH_SEPARATOR) + 1;
      // Create the 2600 game console
      theConsole = new Console(image, size, justfilename, theEvent, propertiesSet, sound);
      // Free the image since we don't need it any longer
      delete[] image;

      DiveMap(theConsole->mediaSource().currentFrameBuffer());

//      theConsole->mediaSource().update();

      NewGamePrep();
      for(;;)
      {
         if(theQuitIndicator)
         {
            break;
         }
         theConsole->mediaSource().update();
         updateDisplay();
         handleEvents();
      }
      DiveUnMap();
      delete[] theConsole;

quitstella:
      SaveThrottle();
      if (newgame == FALSE) break; // quit if no new game desired.
      theQuitIndicator = FALSE;
      newgame = FALSE;
   }
   CloseWindow();
}

