/////////////////////////////////////////////////////////////////////////////
// $Id:$
//////////////////////////////////////////////////////////////////////////////
//                                  Stella
//                        "An Atari 2600 Simulator"
//                         Copyright (c) 1995,1996
//                            Bradford W. Mott
//////////////////////////////////////////////////////////////////////////////
//
// SoundOS2
//
//   This class implements the sound API for the OS/2 operating system.
//
// October 4,1996
//
//////////////////////////////////////////////////////////////////////////////
// $Log:$
//////////////////////////////////////////////////////////////////////////////

#include <string.h>
#include <stdlib.h>
#include "SndOS2.hxx"

// from rsynth/2's hplay.c
#include <stdio.h>
#include <float.h>
#define INCL_WINDIALOGS   // OS/2 include for Window Dialog
#include <os2.h>
#define  INCL_OS2MM
#include <os2me.h>

extern int buflen = 2048;           // 2048 best so far
extern ULONG TIAfreq = 31400;
extern ULONG AUDIOfreq = 31400;
extern int AudioOK = true;

unsigned char *Buf;
unsigned char * curplaybuf;

//////////////////////////////////////////////////////////////////////////////
// Constructor
//////////////////////////////////////////////////////////////////////////////
SoundOS2::SoundOS2()
    : myDisabled(false),
      myMute(false)
{
  // Initialize to impossible values so they will be reset
  // on the first call to the set method
  myAUDC0 = myAUDC1 = myAUDF0 = myAUDF1 = myAUDV0 = myAUDV1 = 255;
  Buf = NULL;
  curplaybuf = NULL;

  if (AudioOK == false)
  {
    myDisabled = true;
    return; // limit to only one warning per Stella run
  }

  if (audio_init() == true)
    myDisabled = false;
  else
  {
    myDisabled = true;
    AudioOK = false;
  }
}

//////////////////////////////////////////////////////////////////////////////
// Destructor
//////////////////////////////////////////////////////////////////////////////
SoundOS2::~SoundOS2()
{
  if(!myDisabled)
     audio_close();
  curplaybuf = NULL;
}

//////////////////////////////////////////////////////////////////////////////
// Set the given sound register to the specified value
//////////////////////////////////////////////////////////////////////////////
void SoundOS2::set(Sound::Register reg, uInt8 value)
{
  static int soundreg;
  // Return if I'm currently disabled or if the stella-sound process has died
  if(myDisabled) return;
  if(AudioOK == false) return;

  uInt8  command;

  switch(reg)
  {
    case AUDC0:
    {
      if(myAUDC0 != (value & 0x0f))
      {
        myAUDC0 = (value & 0x0f);
        command = myAUDC0 | 0x00;
      }
      else
      {
        return;
      }
      soundreg = 0x15; // removed in 1.1 ?
      break;
    }

    case AUDC1:
    {
      if(myAUDC1 != (value & 0x0f))
      {
        myAUDC1 = (value & 0x0f);
        command = myAUDC1 | 0x20;
      }
      else
      {
        return;
      }
      soundreg = 0x16;
      break;
    }

    case AUDF0:
    {
      if(myAUDF0 != (value & 0x1f))
      {
        myAUDF0 = (value & 0x1f);
        command = myAUDF0 | 0x40;
      }
      else
      {
        return;
      }
      soundreg = 0x17;
      break;
    }

    case AUDF1:
    {
      if(myAUDF1 != (value & 0x1f))
      {
        myAUDF1 = (value & 0x1f);
        command = myAUDF1 | 0x60;
      }
      else
      {
        return;
      }
      soundreg = 0x18;
      break;
    }

    case AUDV0:
    {
      if(myAUDV0 != (value & 0x0f))
      {
        myAUDV0 = (value & 0x0f);
        command = myAUDV0 | 0x80;
      }
      else
      {
        return;
      }
      soundreg = 0x19;
      break;
    }

    case AUDV1:
    {
      if(myAUDV1 != (value & 0x0f))
      {
        myAUDV1 = (value & 0x0f);
        command = myAUDV1 | 0xA0;
      }
      else
      {
        return;
      }
      soundreg = 0x1a;
      break;
    }

    default:
    {
      return;
      break;
    }
  }

  Update_tia_sound(soundreg, value);
//  Tia_process(buffer[curbuf],buflen); now in video update routine
  audio_play();

}


void SoundOS2::mute(bool state)
{
  // Return if I'm currently disabled or if the stella-sound process has died
  if (myDisabled)
  {
    myDisabled = true;
    return;
  }
  myMute = state;
  uInt8 command;
  if(myMute)
  {
    // Setup the mute enable command
    command = 0x01 | 0xE0;
  }
  else
  {
    // Setup the mute disable command
    command = 0x00 | 0xE0;
  }
  // Send sound command to the stella-sound process
//  write(myFd, &command, 1);
}





/* PlayList Entry */

typedef struct _PLE {
    ULONG operation;
    ULONG operand1;
    ULONG operand2;
    ULONG operand3;
}PLE;


long samp_rate  = AUDIOfreq;
long bits = 8;
int quiet = FALSE;
int chipmunk = FALSE;
MCI_OPEN_PARMS mop;
PLE playlist[4];
MCI_PLAY_PARMS mpp;
MCI_GENERIC_PARMS mgp;
MCI_WAVE_SET_PARMS wsp;
ULONG rc;


void mci_err(ULONG rc)
{
    const rsize = 128;
    char rbuff[rsize];
    char Text[1000];
    char Title[100];
    strcpy(Title, "MCI Error has occurred!");

    ULONG rc2 = mciGetErrorString(rc,      // error code
                                  rbuff,   // return buffer
                                  rsize);  // rbuff size
    if (rc2 == MCIERR_SUCCESS)
        sprintf(Text,"Error: %s",rbuff);
    else
        sprintf(Text,"Error #: %d", rc);
    WinMessageBox( HWND_DESKTOP, HWND_DESKTOP, Text, Title, 0, MB_OK | MB_INFORMATION );
}

int audio_init(void)
{
//  int rate;
//  rate = 8;
  samp_rate = AUDIOfreq;
  bits = 8;  // bits per sample

  Buf = (unsigned char*) malloc(buflen);
  memset(Buf,0,buflen);

  playlist[0].operation = DATA_OPERATION;
  playlist[0].operand1  = (long) Buf;
  playlist[0].operand2  = buflen;  // size of sample
  playlist[0].operand3  = 0;

  playlist[1].operation = BRANCH_OPERATION;
  playlist[1].operand1  = 0;
  playlist[1].operand2  = 0;  // goto playlist[0]
  playlist[1].operand3  = 0;

  playlist[2].operation = EXIT_OPERATION;

  curplaybuf = Buf;

  mop.hwndCallback   = 0;
  mop.usDeviceID     = 0;
  mop.pszDeviceType  = MCI_DEVTYPE_WAVEFORM_AUDIO_NAME;
  mop.pszElementName = (char *)&playlist[0];

  rc = mciSendCommand(0,
                       MCI_OPEN,            // open message
                       MCI_WAIT | MCI_OPEN_SHAREABLE | // message flags
                       MCI_OPEN_PLAYLIST,
                       &mop,                // parameters
                       0);
  if (rc != MCIERR_SUCCESS)
  {
     mci_err(rc);
     return FALSE;
  }

  // set device parameters
   wsp.hwndCallback    = 0;
   wsp.ulSamplesPerSec = samp_rate;
   wsp.usBitsPerSample = bits;
   rc = mciSendCommand(mop.usDeviceID,
                       MCI_SET,
                       MCI_WAIT |
                       MCI_WAVE_SET_SAMPLESPERSEC |
                       MCI_WAVE_SET_BITSPERSAMPLE,
                       &wsp,
                       0);

    if (rc != MCIERR_SUCCESS)
    {
       mci_err(rc);
       return FALSE;
    }

    mpp.hwndCallback = 0;
    rc = mciSendCommand(mop.usDeviceID,
                        MCI_PLAY,
                        0,
                        &mpp,
                        0);
    if (rc != MCIERR_SUCCESS)
    {
      mci_err(rc);
      return FALSE;
    }



//  Tia_sound_init(TIAfreq,AUDIOfreq);
  Tia_sound_init(AUDIOfreq, AUDIOfreq);
  return TRUE;
}


void audio_play()
{

}

void audio_close(void)
{
  playlist[1].operand2 = 2;

  mgp.hwndCallback = 0;
  rc = mciSendCommand(mop.usDeviceID, MCI_CLOSE, MCI_WAIT, &mgp, 0);
  if (rc != MCIERR_SUCCESS) mci_err(rc);

  if (Buf) free(Buf);
  Buf = NULL;
  curplaybuf = NULL;
}
